from dataclasses import dataclass
from typing import Union

from ngohub.models.core import BaseDataclass
from ngohub.models.organization import OrganizationApplication
from ngohub.models.user import User


@dataclass
class CheckOrganizationUserApplication(BaseDataclass):
    user: Union[User, None]
    application: Union[OrganizationApplication, None]
    has_access: Union[bool, None]

    def __init__(self) -> None:
        self.user = None
        self.application = None
        self.has_access = None



import http
import json
import logging
import socket
import ssl
import urllib.parse
from http.client import HTTPResponse, HTTPSConnection
from typing import Dict, Optional, Tuple, Type

from ngohub.exceptions import HubBadRequestException, HubDecodeException, HubHTTPException

logger = logging.getLogger(__name__)


def retry(attempts: int, exceptions: Tuple[Type[Exception]]):
    """
    Retry Decorator
    Retries the wrapped function/method `times` times if the exceptions listed in ``exceptions`` are thrown

    :param attempts: The number of times to repeat the wrapped function/method
    :param exceptions: Lists of exceptions that trigger a retry attempt
    """

    def decorator(func):
        def new_function(*args, **kwargs):
            attempt = 0
            while attempt < attempts:
                try:
                    return func(*args, **kwargs)
                except exceptions:
                    print("Exception thrown when attempting to run %s, attempt " "%d of %d" % (func, attempt, attempts))
                    attempt += 1
            return func(*args, **kwargs)

        return new_function

    return decorator


class HTTPClientResponse:
    """
    HTTP responses with some helper methods
    """

    def __init__(self, raw_response: HTTPResponse):
        self.raw_response: HTTPResponse = raw_response

    def to_str(self) -> str:
        try:
            string_response: str = self.raw_response.read().decode("utf-8")
        except UnicodeDecodeError:
            raise HubDecodeException(f"Failed to decode response: {self.raw_response.read()}")

        return string_response

    def to_dict(self) -> Dict:
        string_response: str = self.to_str()

        try:
            dict_response = json.loads(string_response)
        except json.JSONDecodeError:
            raise HubDecodeException(f"Failed to decode JSON response: {self.raw_response.read()}")

        return dict_response


class HTTPClient:
    """
    HTTP client for interacting with an HTTP API
    """

    def __init__(self, api_base_url: str, *, auth_type="Bearer", auth_header="Authorization"):
        self.api_base_url = api_base_url or ""
        self.auth_type = auth_type
        self.auth_header = auth_header

    @retry(attempts=3, exceptions=(HubBadRequestException,))
    def _api_request(
        self,
        request_method: str,
        path: str,
        token: str,
        params: Optional[Dict],
    ) -> HTTPClientResponse:
        """
        Perform a request to the NGO Hub API and return a JSON response, or raise HubHTTPException
        """
        if not self.api_base_url:
            raise ValueError("The API base URL cannot be empty")

        if not path.startswith("/"):
            path = f"/{path}"

        conn: HTTPSConnection = http.client.HTTPSConnection(self.api_base_url)

        headers: Dict = {
            "Content-Type": "application/json",
        }
        if token:
            headers[self.auth_header] = f"{self.auth_type} {token}"

        encoded_params = None
        if params:
            encoded_params = urllib.parse.urlencode(params)

        try:
            conn.request(method=request_method, url=path, body=encoded_params, headers=headers)
        except socket.gaierror as e:
            raise HubBadRequestException(
                message=f"Failed to make request to '{path}': {e}",
                path=path,
            )
        except ssl.SSLEOFError as e:
            raise HubBadRequestException(
                message=f"Failed to make request to '{path}': {e}",
                path=path,
            )

        try:
            response: HTTPResponse = conn.getresponse()
        except ConnectionError as e:
            raise HubBadRequestException(
                message=f"Failed to get response from '{path}': {e}",
                path=path,
            )

        if response.status != http.HTTPStatus.OK:
            logger.info(path)
            raise HubHTTPException(
                message=f"{response.status} while retrieving '{path}'. Reason: {response.reason}",
                status_code=response.status,
                path=path,
                reason=response.reason,
            )

        return HTTPClientResponse(response)

    def api_get(self, path: str, token: str = None) -> HTTPClientResponse:
        return self._api_request("GET", path, token, params=None)

    def api_post(self, path: str, params: Dict, token: str = None) -> HTTPClientResponse:
        return self._api_request("POST", path, token, params)

    def api_patch(self, path: str, params: Dict, token: str = None) -> HTTPClientResponse:
        return self._api_request("PATCH", path, token, params)

    def api_delete(self, path: str, token: str = None) -> HTTPClientResponse:
        return self._api_request("DELETE", path, token, params=None)



from typing import Any, Dict, List

from ngohub.models.locations import City, CityBase, County, Region
from ngohub.models.nomenclatures import Domain
from ngohub.models.organization import (
    Application,
    Organization,
    OrganizationActivity,
    OrganizationApplication,
    OrganizationContact,
    OrganizationDirector,
    OrganizationFinancial,
    OrganizationGeneral,
    OrganizationInvestors,
    OrganizationLegal,
    OrganizationLegalReprezentative,
    OrganizationPartners,
    OrganizationReport,
    OrganizationReports,
)
from ngohub.normalization.processors import (
    camel_to_snake_case_dictionary,
    camel_to_snake_case_dictionary_list,
    convert_date,
)


def _normalize_city(city_data: Dict) -> City:
    city_data["county"] = County(**city_data["county"])

    normal_data = City(**city_data)

    return normal_data


def _normalize_organization_general(org_general_data: Dict) -> OrganizationGeneral:
    org_general_data["contact"] = OrganizationContact(**org_general_data["contact"])

    org_general_data["city"] = CityBase(**org_general_data["city"])
    org_general_data["county"] = County(**org_general_data["county"])

    if org_general_data["organization_city"]:
        org_general_data["organization_city"] = CityBase(**org_general_data["organization_city"])
    if org_general_data["organization_county"]:
        org_general_data["organization_county"] = County(**org_general_data["organization_county"])

    org_general = OrganizationGeneral(**org_general_data)

    return org_general


def _normalize_organization_activity(org_activity_data: Dict) -> OrganizationActivity:
    org_domains: List[Domain] = []
    org_regions: List[Region] = []
    org_cities: List[City] = []

    for domain in org_activity_data["domains"]:
        org_domains.append(Domain(**domain))
    for region in org_activity_data["regions"]:
        org_regions.append(Region(**region))
    for city in org_activity_data["cities"]:
        org_cities.append(_normalize_city(city))

    org_activity_data["domains"] = org_domains
    org_activity_data["regions"] = org_regions
    org_activity_data["cities"] = org_cities

    normal_data = OrganizationActivity(**org_activity_data)

    return normal_data


def _normalize_organization_legal(org_legal_data: Dict) -> OrganizationLegal:
    org_directors: List[OrganizationDirector] = []
    for director in org_legal_data["directors"]:
        org_directors.append(OrganizationDirector(**director))

    org_legal_data["directors"] = org_directors
    org_legal_data["legal_reprezentative"] = OrganizationLegalReprezentative(**org_legal_data["legal_reprezentative"])

    normal_data = OrganizationLegal(**org_legal_data)

    return normal_data


def _normalize_organization_financial(org_financial_data_set: List[Dict]) -> List[OrganizationFinancial]:
    normal_data = [OrganizationFinancial(**org_financial_data) for org_financial_data in org_financial_data_set]

    return normal_data


def _normalize_organization_report(org_report_data: Dict) -> OrganizationReport:
    org_reports: List[OrganizationReports] = []
    org_partners: List[OrganizationPartners] = []
    org_investors: List[OrganizationInvestors] = []

    for report in org_report_data.get("reports"):
        org_reports.append(OrganizationReports(**report))

    for partner in org_report_data.get("partners"):
        org_partners.append(OrganizationPartners(**partner))

    for investor in org_report_data.get("investors"):
        org_investors.append(OrganizationInvestors(**investor))

    org_report_data["reports"] = org_reports
    org_report_data["partners"] = org_partners
    org_report_data["investors"] = org_investors

    normal_data = OrganizationReport(**org_report_data)

    return normal_data


def normalize_organization_data(org_data: Dict) -> Organization:
    snake_case: Dict[str, any] = camel_to_snake_case_dictionary(
        org_data,
        overrides={
            "isPublicIntrestOrganization": "is_public_interest_organization",
            "synched_anaf": "synced_anaf",
            "organizationGeneral": "general_data",
            "organizationActivity": "activity_data",
            "organizationLegal": "legal_data",
            "organizationFinancial": "financial_data",
            "organizationReport": "report_data",
        },
        cast_values={"created_on": convert_date, "updated_on": convert_date},
    )

    snake_case["general_data"] = _normalize_organization_general(snake_case["general_data"])
    snake_case["activity_data"] = _normalize_organization_activity(snake_case["activity_data"])
    snake_case["legal_data"] = _normalize_organization_legal(snake_case["legal_data"])
    snake_case["financial_data"] = _normalize_organization_financial(snake_case["financial_data"])
    snake_case["report_data"] = _normalize_organization_report(snake_case["report_data"])

    normal_data = Organization(**snake_case)

    return normal_data


def normalize_organization_applications(org_data_set: List[Dict[str, Any]]) -> List[OrganizationApplication]:
    snake_case_response: List[Dict[str, any]] = camel_to_snake_case_dictionary_list(
        org_data_set,
        overrides={"ongStatus": "ngo_status"},
        cast_values={"created_on": convert_date},
    )

    normal_data: List[OrganizationApplication] = [
        OrganizationApplication(**org_data) for org_data in snake_case_response
    ]

    return normal_data


def normalize_application_list(application_data_set: List[Dict]) -> List[Application]:
    snake_case_response: List[Dict[str, any]] = camel_to_snake_case_dictionary_list(application_data_set)

    normal_data: List[Application] = [Application(**app_data) for app_data in snake_case_response]

    return normal_data



from dataclasses import dataclass
from datetime import datetime
from typing import Union

from ngohub.models.core import BaseDataclass
from ngohub.models.organization import OrganizationBase


@dataclass
class BaseUser(BaseDataclass):
    id: int
    created_on: datetime
    updated_on: datetime
    cognito_id: str
    name: str
    email: str
    phone: str
    role: str
    status: str


@dataclass
class User(BaseUser):
    organization_id: int


@dataclass
class UserProfile(BaseUser):
    organization: Union[OrganizationBase, None]



from dataclasses import dataclass
from datetime import datetime

from ngohub.models.core import BaseDataclass


@dataclass
class Region(BaseDataclass):
    id: int
    name: str
    created_on: datetime
    updated_on: datetime


@dataclass
class County(BaseDataclass):
    id: int
    name: str
    abbreviation: str
    region_id: int
    created_on: datetime


@dataclass
class CityBase(BaseDataclass):
    id: int
    name: str
    created_on: datetime
    county_id: int


@dataclass
class City(CityBase):
    county: County



import re
from datetime import datetime
from re import Pattern
from typing import Any, Dict, List, Optional

camel_case_to_snake_case_pattern: Optional[Pattern[str]] = None


def _compile_camel_to_snake_case_pattern() -> Pattern[str]:
    global camel_case_to_snake_case_pattern

    if not camel_case_to_snake_case_pattern:
        camel_case_to_snake_case_pattern = re.compile(
            r"""
                (?<=[a-z])      # preceded by lowercase
                (?=[A-Z])       # followed by uppercase
                |               #   OR
                (?<=[A-Z])      # preceded by uppercase
                (?=[A-Z][a-z])  # followed by uppercase, then lowercase
            """,
            re.X,
        )

    return camel_case_to_snake_case_pattern


def camel_to_snake_case(input_string: str) -> str:
    pattern: Pattern[str] = _compile_camel_to_snake_case_pattern()

    return pattern.sub("_", input_string).lower()


def camel_to_snake_case_dictionary(
    input_dict: Dict[str, Any],
    overrides: Dict[str, Any] = None,
    cast_values: Dict[str, Any] = None,
) -> Dict[str, Any]:
    output_dict: Dict[str, Any] = {}

    for old_key, value in input_dict.items():
        if overrides and old_key in overrides:
            new_key = overrides[old_key]
        else:
            new_key = camel_to_snake_case(old_key)

        if new_key in output_dict:
            raise ValueError(f"Cannot normalize {old_key} to {new_key} because { new_key} already exists")

        if cast_values and new_key in cast_values:
            value = cast_values[new_key](value)
        elif isinstance(value, dict):
            value = camel_to_snake_case_dictionary(value, overrides, cast_values)
        elif isinstance(value, list) and value and isinstance(value[0], dict):
            value = camel_to_snake_case_dictionary_list(value, overrides, cast_values)

        output_dict[new_key] = value

    return output_dict


def camel_to_snake_case_dictionary_list(
    input_list: List[Dict[str, Any]],
    overrides: Dict[str, Any] = None,
    cast_values: Dict[str, Any] = None,
) -> List[Dict[str, Any]]:
    return [camel_to_snake_case_dictionary(item, overrides, cast_values) for item in input_list]


def convert_date(date: str) -> Optional[datetime]:
    if not date:
        return None

    return datetime.fromisoformat(date)



from dataclasses import dataclass
from datetime import datetime

from ngohub.models.core import BaseDataclass


@dataclass
class Domain(BaseDataclass):
    id: int
    name: str
    created_on: datetime
    updated_on: datetime



from typing import Any, Dict

from ngohub.models.organization import OrganizationBase
from ngohub.models.user import User, UserProfile
from ngohub.normalization.processors import camel_to_snake_case_dictionary


def normalize_user(user_data: Dict[str, Any]) -> User:
    normal_data = User(
        id=user_data["id"],
        created_on=user_data["createdOn"],
        updated_on=user_data["updatedOn"],
        cognito_id=user_data["cognitoId"],
        name=user_data["name"],
        email=user_data["email"],
        phone=user_data["phone"],
        role=user_data["role"],
        status=user_data["status"],
        organization_id=user_data["organizationId"],
    )

    return normal_data


def normalize_user_profile(user_data: Dict[str, Any]) -> UserProfile:
    snake_case: Dict[str, any] = camel_to_snake_case_dictionary(user_data)

    if snake_case["organization"]:
        snake_case["organization"] = OrganizationBase(**snake_case["organization"])

    normal_data = UserProfile(**snake_case)

    return normal_data



class HubException(Exception):
    """The base exception for all Hub issues"""

    pass


class OrganizationException(HubException):
    """The base exception for all Hub Organization issues"""

    pass


class ClosedOrganizationRegistrationException(OrganizationException):
    """New organizations cannot be registered anymore"""

    pass


class DisabledOrganizationException(OrganizationException):
    """The requested organization has been disabled from the platform"""

    pass


class DuplicateOrganizationException(OrganizationException):
    """An organization with the same NGO Hub ID already exists"""

    pass


class MissingOrganizationException(OrganizationException):
    """The requested organization does not exist"""

    pass


class UserException(HubException):
    """The base exception for all Hub User issues"""

    pass


class MissingUserException(UserException):
    """The requested user does not exist"""

    pass


class HubHTTPException(HubException):
    """The base exception for all Hub HTTP/network issues"""

    def __init__(self, message: str, status_code: int, path: str, reason: str):
        self.message = message
        self.status_code = status_code
        self.path = path
        self.reason = reason

        super().__init__(message)


class HubBadRequestException(HubHTTPException):
    """The request was malformed"""

    def __init__(self, message: str, path: str):
        super().__init__(message, 400, path, "Bad request")


class HubDecodeException(HubHTTPException):
    """Failed to decode response"""

    def __init__(self, message: str):
        super().__init__(message, 500, "", "Internal server error")



from dataclasses import dataclass
from datetime import datetime

from ngohub.models.core import BaseDataclass


@dataclass
class BaseEntity(BaseDataclass):
    id = int
    created_on = datetime
    updated_on = datetime
    name = str
    abbreviation = str


@dataclass
class Federation(BaseEntity): ...


@dataclass
class Coalition(BaseEntity): ...



from abc import ABC, abstractmethod
from typing import Any, Dict, List, Optional, Union
from urllib.parse import urlencode

from ngohub.exceptions import HubHTTPException, MissingUserException
from ngohub.models.checks import CheckOrganizationUserApplication
from ngohub.models.organization import Application, Organization, OrganizationApplication
from ngohub.models.public import Version
from ngohub.models.user import User, UserProfile
from ngohub.network import HTTPClient, HTTPClientResponse
from ngohub.normalization.organization import (
    normalize_application_list,
    normalize_organization_applications,
    normalize_organization_data,
)
from ngohub.normalization.public import normalize_version
from ngohub.normalization.user import normalize_user, normalize_user_profile


class BaseHub(ABC):
    """
    Abstract class used to define all the required methods for a hub interface
    """

    @abstractmethod
    def __init__(self, api_base_url: str) -> None:
        self.api_base_url: str = api_base_url or ""


class NGOHubRaw(BaseHub):
    def __init__(self, api_base_url: str) -> None:
        super().__init__(api_base_url)

        self.client: HTTPClient = HTTPClient(self.api_base_url)

    def get_raw_health(self) -> str:
        response: HTTPClientResponse = self.client.api_get("/health/")

        return response.to_str()

    def get_raw_version(self) -> Dict[str, str]:
        response: HTTPClientResponse = self.client.api_get("/version/")

        return response.to_dict()

    def get_raw_file_url(self, path: str) -> str:
        url_params: Dict[str, str] = {"path": path}
        response: HTTPClientResponse = self.client.api_get(f"/file?{urlencode(url_params)}")

        return response.to_str()

    def _get_raw_nomenclature(self, nomenclature: str) -> Any:
        response: HTTPClientResponse = self.client.api_get(f"/nomenclatures/{nomenclature}")

        return response.to_dict()

    def get_raw_cities_nomenclatures(
        self, search: str = None, county_id: int = None, city_id: int = None
    ) -> List[Dict[str, Any]]:
        mandatory_params: List[Any] = [search, county_id]
        if all(param is None for param in mandatory_params):
            raise ValueError("Please provide at least one of the following: county_id, search")

        url_params: Dict[str, Any] = {}
        if search:
            url_params["search"] = search
        if county_id:
            url_params["countyId"] = county_id
        if city_id:
            url_params["cityId"] = city_id

        return self._get_raw_nomenclature(f"cities?{urlencode(url_params)}")

    def get_raw_counties_nomenclatures(self) -> List[Dict[str, Any]]:
        return self._get_raw_nomenclature("counties")

    def get_raw_domains_nomenclatures(self):
        return self._get_raw_nomenclature("domains")

    def get_raw_regions_nomenclatures(self):
        return self._get_raw_nomenclature("regions")

    def get_raw_federations_nomenclatures(self):
        return self._get_raw_nomenclature("federations")

    def get_raw_coalitions_nomenclatures(self):
        return self._get_raw_nomenclature("coalitions")

    def get_raw_faculties_nomenclatures(self):
        return self._get_raw_nomenclature("faculties")

    def get_raw_skills_nomenclatures(self):
        return self._get_raw_nomenclature("skills")

    def get_raw_practice_domains_nomenclatures(self):
        return self._get_raw_nomenclature("practice-domains")

    def get_raw_service_domains_nomenclatures(self):
        return self._get_raw_nomenclature("service-domains")

    def get_raw_beneficiaries_nomenclatures(self):
        return self._get_raw_nomenclature("beneficiaries")

    def get_raw_issuers_nomenclatures(self):
        return self._get_raw_nomenclature("issuers")

    def get_raw_profile(self, user_token: str) -> Dict[str, Any]:
        response: HTTPClientResponse = self.client.api_get("/profile/", token=user_token)

        return response.to_dict()

    def get_raw_organization_profile(self, ngo_token: str) -> Dict[str, Any]:
        response: HTTPClientResponse = self.client.api_get("/organization-profile/", token=ngo_token)

        return response.to_dict()

    def get_raw_application_list(self, admin_token: str) -> List[Dict[str, Any]]:
        response: HTTPClientResponse = self.client.api_get("/application/list/", token=admin_token)

        return list(response.to_dict())

    def get_raw_user_organization_applications(self, ngo_token: str) -> List[Dict[str, Any]]:
        response: HTTPClientResponse = self.client.api_get("/organizations/application/", token=ngo_token)

        return list(response.to_dict())

    def get_raw_organization(self, admin_token: str, organization_id: int) -> Dict[str, Any]:
        response: HTTPClientResponse = self.client.api_get(f"/organization/{organization_id}/", token=admin_token)

        return response.to_dict()

    def get_raw_organization_applications(self, admin_token, organization_id) -> List[Dict[str, Any]]:
        response: HTTPClientResponse = self.client.api_get(
            f"/application/organization/{organization_id}/", token=admin_token
        )
        return list(response.to_dict())

    def get_user_raw(self, admin_token: str, user_id: int) -> Dict[str, Any]:
        try:
            response: HTTPClientResponse = self.client.api_get(f"/user/{user_id}/", token=admin_token)
        except HubHTTPException as e:
            if e.status_code == 404:
                raise MissingUserException(f"User with ID {user_id} not found")

            raise e

        return response.to_dict()

    def get_raw_users(
        self,
        admin_token: str,
        organization_id: int,
        limit: int = 1000,
        page: int = 1,
        search: str = None,
        order_by: str = None,
        order_direction: str = None,
        start: str = None,
        end: str = None,
        status: str = None,
        available_apps_ids: List[int] = None,
    ) -> Dict[str, Any]:
        base_url: str = "/user"
        url_params: dict = {"organization_id": organization_id, "limit": limit, "page": page}

        if search:
            url_params["search"] = search

        if order_by:
            url_params["orderBy"] = order_by
        if order_direction and order_direction.upper() in ["ASC", "DESC"]:
            url_params["orderDirection"] = order_direction.upper()

        if start:
            url_params["start"] = start
        if end:
            url_params["end"] = end

        if status and status.lower() in ["active", "pending", "restricted"]:
            url_params["status"] = status.lower()

        if available_apps_ids:
            url_params["availableAppsIDs"] = [str(app_id) for app_id in available_apps_ids]

        request_url: str = f"{base_url}?{urlencode(url_params)}"
        response: HTTPClientResponse = self.client.api_get(request_url, token=admin_token)

        return response.to_dict()


class NGOHub(NGOHubRaw):
    def __init__(self, api_base_url: str) -> None:
        super().__init__(api_base_url)

    def is_healthy(self) -> bool:
        return bool(self.get_raw_health() == "OK")

    def get_version(self) -> Version:
        return normalize_version(self.get_raw_version())

    def get_file_url(self, path: str) -> str:
        return self.get_raw_file_url(path)

    def get_profile(self, user_token: str) -> UserProfile:
        response: Dict[str, Any] = self.get_raw_profile(user_token=user_token)

        return normalize_user_profile(response)

    # Organization related methods
    def get_organization_profile(self, ngo_token: str) -> Organization:
        response: Dict[str, Any] = self.get_raw_organization_profile(ngo_token)

        return normalize_organization_data(response)

    def get_user_organization_applications(self, ngo_token: str) -> List[OrganizationApplication]:
        response: List[Dict[str, Any]] = self.get_raw_user_organization_applications(ngo_token)

        return normalize_organization_applications(response)

    def check_user_organization_has_application(
        self, ngo_token: str, login_link: str
    ) -> Union[OrganizationApplication, None]:
        organization_applications: List[OrganizationApplication] = self.get_user_organization_applications(ngo_token)

        for app in organization_applications:
            if app.login_link.startswith(login_link) and app.status == "active" and app.ngo_status == "active":
                return app

        return None

    # Admin related methods
    def get_application_list(self, admin_token: str) -> List[Application]:
        response: List[Dict[str, Any]] = self.get_raw_application_list(admin_token)

        return normalize_application_list(response)

    def get_organization(self, admin_token: str, organization_id: int) -> Organization:
        response: Dict[str, Any] = self.get_raw_organization(admin_token=admin_token, organization_id=organization_id)

        return normalize_organization_data(response)

    def get_organization_applications(self, admin_token: str, organization_id: int) -> List[OrganizationApplication]:
        response = self.get_raw_organization_applications(admin_token, organization_id)

        return normalize_organization_applications(response)

    def get_user(self, admin_token: str, user_id: int) -> User:
        response: Dict[str, Any] = self.get_user_raw(admin_token, user_id)

        return normalize_user(response)

    def check_organization_has_application(
        self, admin_token: str, organization_id: int, login_link: str
    ) -> Optional[OrganizationApplication]:
        organization_applications: List[OrganizationApplication] = self.get_organization_applications(
            admin_token, organization_id
        )

        for application in organization_applications:
            if (
                application.login_link.startswith(login_link)
                and application.status == "active"
                and application.ngo_status == "active"
            ):
                return application

        return None

    def _check_user_has_application(
        self, admin_token: str, organization_id: int, user_id: int, response: CheckOrganizationUserApplication
    ) -> CheckOrganizationUserApplication:
        continue_searching: bool = True
        page: int = 1

        searched_application_id = response.application.id

        while continue_searching:
            organization_users: Dict[str, Union[Dict, List]] = self.get_raw_users(
                admin_token=admin_token, organization_id=organization_id, page=page
            )

            response_metadata = organization_users.get("meta", {})
            response_users = organization_users.get("items", [])

            if response_metadata["totalPages"] <= response_metadata["currentPage"]:
                continue_searching = False
            else:
                page += 1

            for user in response_users:
                if user["id"] == int(user_id):
                    response.user = user
                    user_applications: List[Dict[str, Any]] = user["availableAppsIDs"]

                    if searched_application_id in user_applications:
                        response.has_access = True
                        return response

                    response.has_access = False
                    return response

        return response

    def check_organization_user_has_application(
        self,
        admin_token: str,
        organization_id: int,
        user_id: int,
        login_link: str,
    ) -> CheckOrganizationUserApplication:

        response: CheckOrganizationUserApplication = CheckOrganizationUserApplication()

        organization_application: OrganizationApplication = self.check_organization_has_application(
            admin_token, organization_id, login_link
        )
        if not organization_application:
            return response

        response.application = organization_application

        try:
            user: User = self.get_user(admin_token, user_id)
        except MissingUserException:
            return response

        if not user or user.organization_id != int(organization_id):
            return response

        response.user = user

        if user.role == "admin":
            response.has_access = True

            return response

        response = self._check_user_has_application(admin_token, organization_id, user_id, response)

        return response



from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

from ngohub.models.core import BaseDataclass
from ngohub.models.entities import Coalition, Federation
from ngohub.models.locations import City, CityBase, County, Region
from ngohub.models.nomenclatures import Domain


@dataclass
class OrganizationBase(BaseDataclass):
    id: int
    created_on: datetime
    updated_on: datetime
    status: str


@dataclass
class OrganizationContact(BaseDataclass):
    email: str
    phone: str
    full_name: str
    name: str = ""

    def __post_init__(self):
        if not self.name:
            self.name = self.full_name


@dataclass
class OrganizationLegalReprezentative(BaseDataclass):
    id: int
    created_on: datetime
    updated_on: datetime
    full_name: str
    email: str
    phone: str
    role: str


@dataclass
class OrganizationDirector(BaseDataclass):
    id: int
    created_on: datetime
    updated_on: datetime
    full_name: str
    email: str
    phone: str
    role: str


@dataclass
class OrganizationReportFile(BaseDataclass):
    id: int
    created_on: datetime
    updated_on: datetime
    year: int
    status: str


@dataclass
class OrganizationReports(OrganizationReportFile):
    number_of_volunteers: int
    number_of_contractors: int
    report: str


@dataclass
class OrganizationPartners(OrganizationReportFile):
    number_of_partners: int
    path: str


@dataclass
class OrganizationInvestors(OrganizationReportFile):
    number_of_investors: int
    path: str


@dataclass
class AssociationRegistryIssuer(BaseDataclass):
    id: int
    name: str
    created_on: datetime
    updated_on: datetime


@dataclass
class OrganizationGeneral(BaseDataclass):
    id: str
    created_on: datetime
    updated_on: datetime
    name: str
    alias: str
    type: str
    email: str
    phone: str
    year_created: str
    cui: str
    association_registry_number: str
    association_registry_part: str
    association_registry_section: str
    association_registry_issuer_id: str
    national_registry_number: str
    raf_number: str
    short_description: str
    description: str
    address: str
    logo: str
    website: str
    facebook: str
    instagram: str
    twitter: str
    linkedin: str
    tiktok: str
    donation_website: str
    redirect_link: str
    donation_sms: str
    donation_keyword: str
    contact: OrganizationContact
    organization_address: str
    city: CityBase
    county: County
    organization_city: Optional[CityBase]
    organization_county: Optional[County]
    association_registry_issuer: AssociationRegistryIssuer


@dataclass
class OrganizationActivity(BaseDataclass):
    id: int
    created_on: datetime
    updated_on: datetime
    area: str
    is_part_of_federation: bool
    is_part_of_coalition: bool
    is_part_of_international_organization: bool
    international_organization_name: str
    is_social_service_viable: bool
    offers_grants: bool
    is_public_interest_organization: bool
    has_branches: bool
    federations: List[Federation]
    coalitions: List[Coalition]
    domains: List[Domain]
    cities: List[City]
    branches: List[City]
    regions: List[Region]


@dataclass
class OrganizationLegal(BaseDataclass):
    id: int
    created_on: datetime
    updated_on: datetime
    others: str
    organization_statute: str
    non_political_affiliation_file: str
    balance_sheet_file: str
    legal_reprezentative: OrganizationLegalReprezentative
    directors: List[OrganizationDirector]


@dataclass
class OrganizationFinancial(BaseDataclass):
    id: int
    created_on: datetime
    updated_on: datetime
    type: str
    number_of_employees: int
    year: int
    total: int
    status: str
    report_status: str
    synced_anaf: bool
    data: str


@dataclass
class OrganizationReport(BaseDataclass):
    id: int
    created_on: datetime
    updated_on: datetime
    reports: List[OrganizationReports]
    partners: List[OrganizationPartners]
    investors: List[OrganizationInvestors]


@dataclass
class Organization(OrganizationBase):
    general_data: OrganizationGeneral
    activity_data: OrganizationActivity
    legal_data: OrganizationLegal
    financial_data: List[OrganizationFinancial]
    report_data: OrganizationReport


@dataclass
class Application(BaseDataclass):
    id: int
    name: str


@dataclass
class OrganizationApplication(Application):
    logo: str
    short_description: str
    login_link: str
    website: str
    status: str
    ngo_status: str
    created_on: datetime
    type: str
    application_label: str
